using UnityEngine;
using System.Collections;

public class Agent : MonoBehaviour {
	
	public Engine engine;
	public Vector3 directionVoulue;
	public AudioSource sourceSon;
	public bool collisionAvecMur = false;
	
	public float argumentSin;
	
	private bool haut = false;
	
	void Start () {
		engine = (Engine)GameObject.Find("Engine").GetComponent(typeof(Engine));
		
		directionVoulue = new Vector3(0, 0, 0);
		
		argumentSin = ((float) engine.rand.NextDouble()) * 3.1416F;
		
		if(name == "Agent1")
		{
			sourceSon = (AudioSource) GetComponent(typeof(AudioSource));
		}
	}
	
	
	void Update () {
		directionVoulue = directionVoulue + regle1() + regle2() + regle3() + regle4() + regle5();
        limiterVitesse();
		verifierToken();
		regle6();
		transform.position = transform.position + (directionVoulue * Time.deltaTime * 40);
		//la normalisation de la vitesse se fait avec time.deltatime qui est le nombre de ms depuis la dernière frame, multiplié par 40 pour garder la même vitesse de boid qu'à 40fps
		
		if(collisionAvecMur)
		{
			GenererCollision();
			collisionAvecMur = false;
		}
		
		argumentSin += Time.deltaTime;
		float scaleBalle = Mathf.Sin(argumentSin) + 1;
		if(scaleBalle < 0.3F)
		{
			scaleBalle = 0.3F;
		}
		transform.localScale = new Vector3(scaleBalle, scaleBalle, scaleBalle);
	}
	
	
	/// <summary>
	/// Quand on clique dessus le boid
	/// </summary>
	void OnMouseDown()
	{
		if(name != "Agent1")
		{
			engine.detruireBoid(this.name);
		}
	}
	
	
	/// <summary>
	/// Rapprochement. Voler vers les autres.
	/// </summary>
	private Vector3 regle1()
	{
		Vector3 ADate = new Vector3(0, 0, 0);
		foreach(GameObject unBoid in engine.lesAgents.Values)
		{
			if(unBoid.name != name)
			{
				ADate = ADate + unBoid.transform.position;
			}
		}
		
		ADate = ADate / (engine.lesAgents.Count-1);
		
		return (ADate - transform.position) / engine.facteurRapprochement;
	}
	
	
	/// <summary>
	/// Éloignement. Garder une distance avec les autres.
	/// </summary>
	private Vector3 regle2()
	{
		Vector3 ADate = new Vector3(0, 0, 0);
		foreach(GameObject unBoid in engine.lesAgents.Values)
		{
			if(unBoid.name != name)
			{
				if(Vector3.Distance(unBoid.transform.position, transform.position) < engine.facteurCollision)
				{
					GenererCollision();
					ADate = ADate - ((unBoid.transform.position - transform.position));	
				}
			}
		}
		return ADate;
	}
	
	
	/// <summary>
	/// Cohésion. Garder la même direction que les autres.
	/// </summary>
	private Vector3 regle3()
	{
		Vector3 ADate = new Vector3(0, 0, 0);
		foreach(GameObject unBoid in engine.lesAgents.Values)
		{
			if(unBoid.name != name)
			{
				ADate = ADate + ((Agent)unBoid.GetComponent(typeof(Agent))).directionVoulue;
			}
		}
		
		ADate = ADate / (engine.lesAgents.Count-1);
		
		
		return (ADate - directionVoulue) / engine.facteurCohesion;
	}
	
	
	/// <summary>
	/// Collisionneur avec l'environnement
	/// </summary>
	private Vector3 regle4()
	{
		Vector3 ADate = new Vector3(0, 0, 0);
		
		if(transform.position.x <= -(engine.demiLargeurScene -2))
		{
			ADate = ADate + new Vector3(engine.facteurCollisionMur, 0, 0);
			collisionAvecMur = true;
		}
		else if(transform.position.x >= (engine.demiLargeurScene -2)) {
			ADate = ADate + new Vector3(-engine.facteurCollisionMur, 0, 0);
			collisionAvecMur = true;
		}
		
		if(transform.position.y >= (engine.demiHauteurScene -2)) {
			ADate = ADate + new Vector3(0, -engine.facteurCollisionMur, 0);
			collisionAvecMur = true;
		}
		else if(transform.position.y <= -(engine.demiHauteurScene -2)) {
			ADate = ADate + new Vector3(0, engine.facteurCollisionMur, 0);
			collisionAvecMur = true;
		}
		
		return ADate;
	}
	
	
	/// <summary>
	/// Fonction de hasard, j'ai remarqué que les boids finissent par aller au bout d'un temps très grand dans un moton qui bouge plus, donc je mets légèrement du hasard dans le déplacement
	/// </summary>
	private Vector3	regle5()
	{
		Vector3 ADate = new Vector3(0, 0, 0);
		
		if(engine.rand.Next(1,1500) == 1)//environ une frame sur x seulement
		{
			ADate = new Vector3((float)engine.rand.NextDouble() * 1F -0.5F, (float)engine.rand.NextDouble() * 1F -0.5F, 0F);
		}
		
		return ADate;
	}

	
	/// <summary>
	/// Fonction de champ de force
	/// </summary>
	private void regle6()
	{		
		if(Vector3.Distance(new Vector3(0,0,0), transform.position) < 10)
		{
			if(haut == false)
			{
				transform.position = new Vector3(transform.position.x, transform.position.y + 0.5F, transform.position.z);
			}
			else
			{
				transform.position = new Vector3(transform.position.x, transform.position.y - 0.5F, transform.position.z);
			}
		}
	}

	
	/// <summary>
	/// Limite la vitesse du boid en limitant la direction voulu
	/// </summary>
    private void limiterVitesse()
    {
        float normedirectionVoulue = directionVoulue.magnitude;
        if (normedirectionVoulue > engine.vitesseMax)
        {
            directionVoulue = (directionVoulue / normedirectionVoulue) * engine.vitesseMax;
        }
    }
		
	private void GenererCollision()
	{
		engine.GenererCollision();
		
		if(name == "Agent1" && engine.tempsDepuisDerniereCollision >= 0.5F)
		{
			if(engine.CameraPremierePersonne3.enabled || engine.CameraTroisiemePersonne4.enabled)
			{
				sourceSon.volume = 0.5F;
			} else {
				sourceSon.volume = 1F;
			}
			sourceSon.Play();
			engine.tempsDepuisDerniereCollision = 0F;
		}
	}
	
	private void verifierToken()
	{
		foreach(string unToken in engine.lesTokens.Keys)
		{
			if(Vector3.Distance(engine.lesTokens[unToken].transform.position, transform.position) < 1.5F)
			{
				engine.detruireTokens(unToken);
				break;
			}
		}
	}

}
