using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Engine : MonoBehaviour {
	
	// Le premier Agent
	public GameObject Agent1;
	
	//les différentes caméras
	public Camera CameraOrtho1;
	public Camera CameraPerspective2;
	public Camera CameraPremierePersonne3;
	public Camera CameraTroisiemePersonne4;
	
	// Les AudioListener des camera
	public AudioListener ListenerCamera1;
	public AudioListener ListenerCamera2;
	public AudioListener ListenerCamera3;
	public AudioListener ListenerCamera4;

    // Pour le HUD
    public GUIText nbAgentsText;
	public GUIText msTrameText;
	public GUIText nbCollisionsText;
	public GUIText nbFPSText;

    public float   intervalleMAJ = 0.5F;
    public int     numeroTrame = 0;
    public float   tramesADate = 0;
    public float   tempsRestantIntervalle = 0F;
    
	public int     nbCollisions = 0;
	
	public bool    HUDEnabled = true;
	
	// Les couleurs
	public Color couleurFond = Color.black;
	public Color couleurAgent1 = Color.white;
	public Color couleurMurs = Color.white;
	
	// Texture de balle
	public Texture balleTexture;
	
	// Texture de balle
	public Texture tokenTexture;
	
	// Gestion des collisions
	public float tempsDepuisDerniereCollision = 0F;
	
	//facteur pour qu'il se met tout ensemble, ralenti les boids
	public float facteurRapprochement;
	
	//à quel distance il y a collision entre deux boids
	public float facteurCollision;
	
	//à quel point ils veulent aller dans la même dirrection, accélère les boids
    public float facteurCohesion;
	
	//de quel distance est retourné dans l'autre direction lors de collision avec le mur
    public float facteurCollisionMur;
	
	//vitesseMaximum
    public float vitesseMax;
	
	//facteur selon le format 16:9 de la scène, doit être pair
    public const int facteur169 = 10;
    public int hauteurScene;
    public int largeurScene;
	//on garde les demis parce qu'il nous les faut souvent
    public int demiHauteurScene;
    public int demiLargeurScene;
	
	//pour du aléatoire
	public System.Random rand = new System.Random();
	
	//un identifiant unique pour chaque boid créer
	public int idAgent = 1;
	
	public SortedList<string, GameObject> lesAgents = new SortedList<string, GameObject>();
	
	public int idToken = 1;
	public SortedList<string, GameObject> lesTokens = new SortedList<string, GameObject>();
	
	// Use this for initialization
	void Start () {
		Agent1.renderer.material.color = couleurAgent1;
		
		generationCamera();		
		generationMur();
		
		GameObject champForce = GameObject.CreatePrimitive(PrimitiveType.Sphere);
		champForce.name = "ChampForce";
		champForce.transform.localScale = new Vector3(10,10,10);
		champForce.transform.position = new Vector3(0, 0, 11F);
		champForce.renderer.material.color = new Color(1F,1F,1F);
		
		//parce que quand on met = en haut, c'est la valeur dans unity qui prend le dessus, donc sa sert à rien de mettre un = en haut, autant ne mettre les valeurs que dans la scène
		facteurRapprochement 	= 1500F;
		facteurCollision 		= 1.5F;
		facteurCohesion			= 10F;
		facteurCollisionMur 	= 1F;
		vitesseMax 				= 1F;
		
		lesAgents.Add("Agent1", GameObject.Find("Agent1")); //le seul agent non procédural
		for(int i = 0; i < 9; i++)
		{
			creerBoid((float)rand.Next(-demiLargeurScene, demiLargeurScene), (float) rand.Next(-demiHauteurScene, demiHauteurScene));	
		}
		
		//on crée des tokens
		for(int i = 0; i < 10; i++)
		{
			creerToken((float)rand.Next(-demiLargeurScene, demiLargeurScene), (float) rand.Next(-demiHauteurScene, demiHauteurScene));	
		}

        tempsRestantIntervalle = intervalleMAJ;
	}
	
	/// <summary>
	/// Génère les murs de la scène à la bonne grandeur
	/// </summary>
	private void generationMur()
	{
		//calcule de la taille de la vue
		hauteurScene = facteur169 * 9;
		largeurScene = facteur169 * 16;
		demiHauteurScene = hauteurScene/2;
		demiLargeurScene = largeurScene/2;
		
		GameObject MurDroit 	= GameObject.Find("MurDroit");
		GameObject MurGauche 	= GameObject.Find("MurGauche");
		GameObject MurHaut  	= GameObject.Find("MurHaut");
		GameObject MurBas 		= GameObject.Find("MurBas");
		
		
		MurDroit.transform.localScale = new Vector3(1, hauteurScene, 1);
		MurDroit.transform.localPosition = new Vector3((float)-(demiLargeurScene-0.5), 0, 0);
		
		MurGauche.transform.localScale = new Vector3(1, hauteurScene, 1);
		MurGauche.transform.localPosition = new Vector3((float)(demiLargeurScene-0.5), 0, 0);
		
		MurHaut.transform.localScale = new Vector3(largeurScene, 1, 1);
		MurHaut.transform.localPosition = new Vector3(0, (float)(demiHauteurScene-0.5), 0);
		
		MurBas.transform.localScale = new Vector3(largeurScene, 1, 1);
		MurBas.transform.localPosition = new Vector3(0, (float)-(demiHauteurScene-0.5), 0);
		
		MurDroit.renderer.material.color  = couleurMurs;
		MurGauche.renderer.material.color = couleurMurs;
		MurHaut.renderer.material.color   = couleurMurs;
		MurBas.renderer.material.color    = couleurMurs;
		
		CameraOrtho1.orthographicSize = demiHauteurScene;
		
	}
	
	/// <summary>
	/// Met les caméras correct dès le départ de la scène
	/// </summary>
	private void generationCamera()
	{
		CameraOrtho1 			 = (Camera)GameObject.Find("CameraOrtho1").GetComponent(typeof(Camera));
		CameraPerspective2 		 = (Camera)GameObject.Find("CameraPerspective2").GetComponent(typeof(Camera));
		CameraPremierePersonne3  = (Camera)GameObject.Find("CameraPremierePersonne3").GetComponent(typeof(Camera));
		CameraTroisiemePersonne4 = (Camera)GameObject.Find("CameraTroisiemePersonne4").GetComponent(typeof(Camera));
		
		CameraOrtho1.backgroundColor             = couleurFond;
		CameraPerspective2.backgroundColor       = couleurFond;
		CameraPremierePersonne3.backgroundColor  = couleurFond;
		CameraTroisiemePersonne4.backgroundColor = couleurFond;
		
		// Instancier les listener des caméras
		ListenerCamera1 = (AudioListener) CameraOrtho1.GetComponent(typeof(AudioListener));
		ListenerCamera2 = (AudioListener) CameraPerspective2.GetComponent(typeof(AudioListener));
		ListenerCamera3 = (AudioListener) CameraPremierePersonne3.GetComponent(typeof(AudioListener));
		ListenerCamera4 = (AudioListener) CameraTroisiemePersonne4.GetComponent(typeof(AudioListener));
		
		ActiverCamera(1);
	}
	
	// Update is called once per frame
	void Update () {
        if(Input.GetKeyDown("1"))
		{
			ActiverCamera(1);
		}
		else if(Input.GetKeyDown("2"))
		{
			ActiverCamera(2);
		}
		else if(Input.GetKeyDown("3"))
		{
			ActiverCamera(3);
		}
		else if(Input.GetKeyDown("4"))
		{
			ActiverCamera(4);
		}
		else if(Input.GetKeyDown("d"))
		{
			creerBoid((float)rand.Next(-demiLargeurScene, demiLargeurScene), (float) rand.Next(-demiHauteurScene, demiHauteurScene));
		}
		else if(Input.GetKeyDown("s"))
		{
			if(lesAgents.Count > 5)
			{
				detruireBoid(lesAgents.Keys[lesAgents.Count-1]);
			}
		}
		
		
		if(Input.GetMouseButtonUp(1) && CameraOrtho1.enabled)
		{
			float sourisX = (Input.mousePosition.x / Screen.width) * largeurScene - demiLargeurScene;
			float sourisY = (Input.mousePosition.y / Screen.height) * hauteurScene - demiHauteurScene;
			
			if(sourisX <= (demiLargeurScene-2) && sourisX >= -(demiLargeurScene-2) && sourisY <= (demiHauteurScene-2) && sourisY >= -(demiHauteurScene-2))
			{
				creerBoid(sourisX, sourisY);
			}
		}
		
        tempsRestantIntervalle -= Time.deltaTime;
        tramesADate += Time.timeScale / Time.deltaTime;
        ++numeroTrame;
        if (tempsRestantIntervalle < 0.0)
        {
            float nbFPS = tramesADate / numeroTrame;
			
			if(HUDEnabled)
			{
	            nbFPSText.text = nbFPS + " Trame/seconde";
				msTrameText.text = 1000/nbFPS + " ms/Trame";
			} else {
				nbFPSText.text = "";
				msTrameText.text = "";
			}

            tempsRestantIntervalle = intervalleMAJ;
            tramesADate = 0.0F;
            numeroTrame = 0;
        }
		
		// HUD
		if(HUDEnabled)
		{
            nbCollisionsText.text = nbCollisions + " collisions";
			nbAgentsText.text = lesAgents.Count + " Agents";
		} else {
			nbCollisionsText.text = "";
			nbAgentsText.text = "";
			nbFPSText.text = "";
			msTrameText.text = "";
		}
		if(Input.GetKeyDown(KeyCode.H))
		{
			if(HUDEnabled)
			{
				HUDEnabled = false;
			} else {
				HUDEnabled = true;
			}
		}
        
		tempsDepuisDerniereCollision += Time.deltaTime;
		
		
		if(rand.Next(1,400) == 1)//environ une frame sur x seulement
		{
			creerToken((float)rand.Next(-demiLargeurScene, demiLargeurScene), (float) rand.Next(-demiHauteurScene, demiHauteurScene));	
		}
	}
	
	/// <summary>
	/// Créer un agent, incrémente le nombre d'agents et l'ajoute dans la liste d'agents
	/// </summary>
	public void creerBoid(float x, float y)
	{
		idAgent++;
		string sonNom = "Agent" + idAgent;
		
		GameObject boid = GameObject.CreatePrimitive(PrimitiveType.Sphere);
		boid.AddComponent("Agent");
		boid.name = sonNom;
		boid.transform.position = new Vector3(x, y, 0);
		boid.renderer.material.mainTexture = balleTexture;
		
		// Assigner une couleur au hasard au boid
		double rouge, vert, bleu;
		rouge = rand.NextDouble();
		vert = rand.NextDouble();
		bleu = rand.NextDouble();
		boid.renderer.material.color = new Color((float) rouge,(float) vert,(float) bleu);
		
		lesAgents.Add(sonNom, boid);
	}
	
	/// <summary>
	/// Créer un token
	/// </summary>
	public void creerToken(float x, float y)
	{
		idToken++;
		string sonNom = "token" + idToken;
		
		GameObject token = GameObject.CreatePrimitive(PrimitiveType.Sphere);
		token.name = sonNom;
		token.transform.position = new Vector3(x, y, 0);
		
		token.transform.localScale = new Vector3(2, 2, 2);
		token.renderer.material.color = new Color(1F,1F,0F);
		
		lesTokens.Add(sonNom, token);
	}
	
	public void ActiverCamera(int p_NumeroCam)
	{
		switch(p_NumeroCam)
		{
		case 1:
			CameraOrtho1.enabled             = true;
			CameraPerspective2.enabled       = false;
			CameraPremierePersonne3.enabled  = false;
			CameraTroisiemePersonne4.enabled = false;
			
			ListenerCamera1.enabled = true;
			ListenerCamera2.enabled = false;
			ListenerCamera3.enabled = false;
			ListenerCamera4.enabled = false;
			break;
		case 2:
			CameraOrtho1.enabled             = false;
			CameraPerspective2.enabled       = true;
			CameraPremierePersonne3.enabled  = false;
			CameraTroisiemePersonne4.enabled = false;
			
			ListenerCamera1.enabled = false;
			ListenerCamera2.enabled = true;
			ListenerCamera3.enabled = false;
			ListenerCamera4.enabled = false;
			break;
		case 3:
			CameraOrtho1.enabled             = false;
			CameraPerspective2.enabled       = false;
			CameraPremierePersonne3.enabled  = true;
			CameraTroisiemePersonne4.enabled = false;
			
			ListenerCamera1.enabled = false;
			ListenerCamera2.enabled = false;
			ListenerCamera3.enabled = true;
			ListenerCamera4.enabled = false;
			break;
		case 4:
			CameraOrtho1.enabled             = false;
			CameraPerspective2.enabled       = false;
			CameraPremierePersonne3.enabled  = false;
			CameraTroisiemePersonne4.enabled = true;
			
			ListenerCamera1.enabled = false;
			ListenerCamera2.enabled = false;
			ListenerCamera3.enabled = false;
			ListenerCamera4.enabled = true;
			break;
		}
	}
	
	/// <summary>
	/// Détruit un gameobjet qui doit être un boid à partir du nom
	/// </summary>
	public void detruireBoid(string name)
	{
		lesAgents.Remove(name);
		detruireObjet(name);
	}
	
	public void detruireTokens(string name)
	{
		lesTokens.Remove(name);
		detruireObjet(name);
	}
	
	/// <summary>
	/// Détruit un gameobject
	/// </summary>
	public void detruireObjet(string name)
	{
		Destroy(GameObject.Find(name));
	}
	
	public void GenererCollision()
	{
		nbCollisions++;
	}
}
